import QtQuick 2.0
import "private"
import QtMultimedia 5.15

import ShaderTransitionView 1.0

Rectangle {
    id: root

    property var shaderEffectOptions: { 
                    "progress": 0.0, 
                    "forward": true
                 }

    property string shaderEffect: "pinwheel"

    property real progress: 0.0
    property real duration: 8.0

	property alias from: textureSource.sourceItem
	property alias to: textureDestination.sourceItem


	signal animationStarted();
    signal animationCompleted();

	property var transitions : inputs.allowedTransitions.split( " " );


	function startTransition( item ) {

		from = to
		to = item
		// no transition if no *from* item or transitioning to *same* item
		if ( (from === null) || (from === to) ) {
			loaderShaderEffect.setSource( "private/ShaderEffectPassTo.qml", shaderEffectOptions )
		} else {
			loaderShaderEffect.setSource( getShader(), shaderEffectOptions )
		}
	}

    function getShader() {

        var retFile = "private/ShaderEffectPassTo.qml"

        if( shaderEffect == "wind" ) {
            retFile = "private/ShaderEffectWind.qml"
        } else if (  shaderEffect == "vertical" ) {
            retFile = "private/ShaderEffectVerticalWind.qml"
        } else if (  shaderEffect == "pinwheel" ) {
            retFile = "private/ShaderEffectPinWheel.qml"
        } else if (  shaderEffect == "circle" ) {
            retFile = "private/ShaderEffectCircleOpen.qml"
        } else if (  shaderEffect == "directional" ) {
            retFile = "private/ShaderEffectDirectionalWipe.qml"
        } else if (  shaderEffect == "radial" ) {
            retFile = "private/ShaderEffectRadialWipe.qml"
        } else if (  shaderEffect == "pixelize" ) {
            retFile = "private/ShaderEffectPixelize.qml"
        } else if (  shaderEffect == "flip" ) {
            retFile = "private/ShaderEffectFlip.qml"
        } else if (  shaderEffect == "fold" ) {
            retFile = "private/ShaderEffectFold.qml"
        } else if (  shaderEffect == "doorway" ) {
            retFile = "private/ShaderEffectDoorway.qml"
        } else if (  shaderEffect == "fade" ) {
            retFile = "private/ShaderEffectFade.qml"
        } else if (  shaderEffect == "black" ) {
            retFile = "private/ShaderEffectFadeColor.qml"
        } else if (  shaderEffect == "morph" ) {
            retFile = "private/ShaderEffectMorph.qml"
        } else if (  shaderEffect == "polkadots" ) {
            retFile = "private/ShaderEffectPolkaDots.qml"
        } else if (  shaderEffect == "squeeze" ) {
            retFile = "private/ShaderEffectSqueeze.qml"
        } else if (  shaderEffect == "hslide" ) {
            retFile = "private/ShaderEffectHorizontalSlide.qml"
        } else if (  shaderEffect == "vslide" ) {
            retFile = "private/ShaderEffectVerticalSlide.qml"
        } else if (  shaderEffect == "cross" ) {
            retFile = "private/ShaderEffectCrossZoom.qml"
        } else if (  shaderEffect == "blur" ) {
            retFile = "private/ShaderEffectLinearBlur.qml"
        } else if (  shaderEffect == "swirl" ) {
            retFile = "private/ShaderEffectSwirl.qml"
        } else if (  shaderEffect == "swap" ) {
            retFile = "private/ShaderEffectSwap.qml"
        } else if (  shaderEffect == "hatch" ) {
            retFile = "private/ShaderEffectCrossHatch.qml"
        }
        return retFile
    }
	
    SequentialAnimation on progress {
        id: animProgress
        running: false

        ScriptAction {
            script: {
                root.animationStarted()
            }
        }

        PropertyAnimation {
            from: 0.0
            to: 1.0
            duration: root.duration
        }

        ScriptAction {
            script: {
				root.progress = 1.0
				root.animationCompleted()
            }
        }


    }

    ShaderEffectSource {
        id: textureSource
        hideSource: true
    }

    ShaderEffectSource {
        id: textureDestination
        hideSource: true
    }

	// effect loader...
    Loader {
        id: loaderShaderEffect
        anchors.fill: parent
        onLoaded: {
            item.srcSampler = textureSource
            item.dstSampler = textureDestination
            root.progress = 0.0
            animProgress.restart()
        }
    }

    Binding {
        target: loaderShaderEffect.item
        property: "progress"
        value: root.progress
    }

    Connections {
        target: inputs
        onAllowedTransitionsChanged: {
            root.transitions = inputs.allowedTransitions.split( " " );
        }
    }

}

